------------------------------------------------------------------------------
-- DebugLog -- Collect and View Debug Logs
------------------------------------------------------------------------------
-- Modules/GUI_debuglog_fmt_tables.lua - Debug Log Format for Tables
--
-- Author: Expelliarm5s / July 2021 / All Rights Reserved
--
-- Version 1.1.12
------------------------------------------------------------------------------
-- luacheck: ignore 212 globals TEMPDLAPI

local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

-- tables Debug Log Format
private.tables = {}
private.tables.colNames = { "ID",   "Type", "Variable",  "Address", "Object", "Name", "Value" }
private.tables.colWidth = { 0.05,   0.08,   0.25,        0.15,      0.1,      0.25,   1 - 0.05 - 0.08 - 0.25 - 0.15 - 0.1 - 0.25,}
private.tables.colFlex  = { "flex", "drop", "flex",      "flex",    "drop",   "flex", "search", }
private.tables.statusText = {
			"Sort by ID",
			"Sort by Type",
			"Sort by Variable",
			"Sort by Address",
			"Sort by Object",
			"Sort by Name",
			"Sort by Value",
		}
private.tables.GetSTData = function(...) return private.GetSTData_tables(...) end

------------------------------------------------------------------------------
-- Get Scrolling Table Data

function private.GetSTData_tables(a, flex, filter)
	private.DebugPrintf("GetSTData_tables(%s, %s, %s)", tostring(a), tostring(flex), tostring(filter))

	-- generate content only if visible
	if not GUI.display or GUI.view ~= a then
		private.DebugPrintf("... nothing done.")
		return
	end

	local content = {}

	-- generate content only if log data present
	addon.debuglog = addon.debuglog or {}
	if not addon.debuglog[a] then
		return content, flex
	end

	-- generate content
	local colHasContent = {}
	local k = 1
	for _, data in pairs(addon.debuglog[a]) do
		local filterMatch = nil  -- = nil -> no filter input = true -> filtered

		-- check dropdown
		for j, col in ipairs(flex) do
			colHasContent[j] = colHasContent[j] or data[j]
			if col == "drop" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						if data[j] and filter[j] == data[j] then
							filterMatch = true
						else
							filterMatch = false
						end
					end
				end
			end
		end

		-- check search
		for j, col in ipairs(flex) do
			if col == "search" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						-- search in all column data
						local found = false
						for m, _ in pairs(data) do
							if data[m] and type(data[m]) == "string" and strfind(strlower(data[m]), strlower(filter[j])) then
								filterMatch = true
								found = true
							end
						 end
						if not found then
							filterMatch = false
						end
					end
				end
			end
		end

		if filterMatch or filterMatch == nil then
			local en = {}

			if data[20] and type(data[20]) == "table" then
				en.en = data
			end

			en.data = {}
			en.data[1] = {k, k}
			for w, _ in ipairs(flex) do
				en.data[w + 1] = {data[w + 1], data[w + 1] or ""}
			end
			tinsert(content, en)
			k = k + 1
		end
	end

	-- disable dropdown if not present
	for j, col in ipairs(flex) do
		if col == "drop" and not colHasContent[j] then
			flex[j] = ""
		end
	end

	return content, flex
end

function private.DebugPrintf(...)
	if TEMPDLAPI and TEMPDLAPI.DebugLog and addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			TEMPDLAPI.DebugLog(addonName, "GUI~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Create a tab to show a table

function GUI.DebugLog_EnableTables(tab, vtab)
	if not GUI.DebugLog_IsFormatRegistered("tables") then
		GUI.DebugLog_RegisterFormat("tables", private.tables)
	end

	-- no tab for empty variables
	if not vtab then
		return
	end

	-- create tab name
	local a
	if type(tab) == "string" and tab ~= "" and tab ~= "nil" then
		a = tab
	else
		a = tostring(vtab)
	end

	-- ensure tab name is unique
	while GUI.guiTabs.timer[a] do
		a = a .. "*"
	end

	private.DebugPrintf("OK~DebugLog_EnableTables(%s, %s) -> (%s, %s)", tostring(tab), tostring(vtab), tostring(a), tostring(vtab))

	-- Event Handlers
	local eventHandlers = {
		OnEnter = function(btn, data)
				if btn.btnCnt then
					GUI:SetStatusLine(GUI.container.titleStatusText[btn.btnCnt])
				else
					if type(data) == "table" and data.en then
						GUI:SetStatusLine("Click to show table in new tab.")
					end

					GameTooltip:SetOwner(btn, "ANCHOR_TOPLEFT")
					GameTooltip:ClearLines()
					for i, n in ipairs(data.data) do
						if i > 1 then
							if n[2] and n[2] ~= "" then
								GameTooltip:AddDoubleLine(private.tables.colNames[i], "|cffffffff" .. tostring(n[1]) .. "|r")
							end
						end
					end
					GameTooltip:Show()
				end
			end,
		OnLeave = function(btn, data)
			if btn.btnCnt then
				GUI:SetStatusLine()
			end
			if type(data) == "table" then
				GameTooltip:Hide()
				if BattlePetTooltip and BattlePetTooltip.Hide then
					BattlePetTooltip:Hide()
				end
				GUI:SetStatusLine()
			end
		end,
		OnClick = function(btn, data, button)
			if not data then return end
			if data.data[2][2] == "table" then
				if data.en and data.en[20] then
					-- peruse in new table tab
					GUI.DebugLog_EnableTables(data.en[3], data.en[20])
				end
			end
		end,
	}

	local fu = {}
	fu[a] = {}
	fu[a].select = function()
		GUI.debugLog_colCtrl[a].w:SetEventHandlers(eventHandlers)
	end

	GUI.DebugLog_CreateTab(a, "tables", fu)

	if addon.DebugLog then
		addon:DebugLog("WARN~Log for '" .. a .. "' created")
	end

	addon.debuglog[a] = addon.debuglog[a] or {}
	if vtab then
		for n, v in pairs(vtab) do
			if strmatch(n, "^Store") or strmatch(n, "^WowToken") or strmatch(n, "^SecureTransfer")
				or strmatch(n, "^ServicesLogout") or strmatch(n, "^AuthChallenge")
				or strmatch(n, "^SimpleCheck") or strmatch(n, "^_DevLoc") then
				local en = {}
				en[3] = tostring(n)
				tinsert(addon.debuglog[a], en)
			else
				local en = {}
				en[2] = type(v)
				en[3] = tostring(n)
				if v and en[2] == "table" then
					en[20] = v -- saved for OnClick handler
					en[4] = strmatch(tostring(v), "table: ([0-9A-Z]+)") or ""
					en[5] = private.tryCall(v.GetObjectType, v) or ""
					en[6] = private.tryCall(v.GetName, v) or ""
				elseif v and en[2] == "function" then
					en[4] = strmatch(tostring(v), "function: ([0-9A-Z]+)") or ""
				elseif v and en[2] == "userdata" then
					en[4] = strmatch(tostring(v), "userdata: ([0-9A-Z]+)") or ""
				else
					en[7] = tostring(v)
				end
				tinsert(addon.debuglog[a], en)
			end
		end
		local meta = getmetatable(vtab)
		if meta and meta.__index then
			meta = meta.__index
			local en = {}
			en[2] = type(meta)
			en[3] = "metatable.__index"
			if meta and en[2] == "table" then
				en[20] = meta -- safed for OnClock handler
				en[4] = strmatch(tostring(meta), "table: ([0-9A-Z]+)") or ""
				en[5] = private.tryCall(meta.GetObjectType, meta) or ""
				en[6] = private.tryCall(meta.GetName, meta) or ""
				tinsert(addon.debuglog[a], en )
			end
		end
	end

	GUI:Load(a)
end

function private.tryCall(func, v)
	if func then
		local status, res = pcall(func, v)
		if status then
			return res
		else
			return "?! " .. tostring(res)
		end
	end
end

-- EOF
